import logging
from typing import Tuple

from pydub import AudioSegment

from cyberpunk.config import get_config


class LocalStorage:
    def __init__(self):
        config = get_config()

        self.base_dir = config.local_storage_base_dir

    def __contains__(self, element):
        return self.contains(element)

    def contains(self, key: str) -> bool:
        return True

    def get_segment(self, key: str) -> Tuple[AudioSegment, str]:
        logging.info(f"pulling key from local storage: {key}")

        location = f"{key}"
        audio_segment = AudioSegment.from_file(
            f"{self.base_dir}{location}",
        )

        return audio_segment, location



"""
Cyberpunk Configuration Module

"""
import os
from typing import Optional


class CyberpunkConfigException(Exception):
    pass


class CyberpunkConfig:
    """Global configuration object for Cyberpunk"""

    def __init__(
        self,
        audio_path: str = "local",
        local_storage_base_dir: Optional[str] = "testdata/",
        local_results_base_dir: Optional[str] = None,
        s3_loader_bucket: Optional[str] = None,
        s3_loader_base_dir: Optional[str] = None,
        s3_storage_bucket: Optional[str] = None,
        s3_storage_base_dir: Optional[str] = None,
        s3_results_bucket: Optional[str] = None,
        s3_results_base_dir: Optional[str] = None,
    ):

        # TODO: validation lol

        # local | s3 | audius
        self.audio_path = audio_path

        if (
            "local" in self.audio_path.split(":")
            and local_storage_base_dir is None
        ):
            raise CyberpunkConfigException(
                "local_storage_base_dir must be configured if `local` in audio_path",
            )

        self.local_storage_base_dir = local_storage_base_dir
        self.local_results_base_dir = local_results_base_dir

        if "s3" in self.audio_path.split(":") and s3_storage_bucket is None:
            raise CyberpunkConfigException(
                "s3_storage_bucket must be configured if `s3` in audio_path",
            )

        self.s3_loader_bucket = s3_loader_bucket
        self.s3_loader_base_dir = s3_loader_base_dir

        self.s3_storage_bucket = s3_storage_bucket
        self.s3_storage_base_dir = s3_storage_base_dir

        self.s3_results_bucket = s3_results_bucket
        self.s3_results_base_dir = s3_results_base_dir

    def __repr__(self):
        return (
            f"CyberpunkConfig ( "
            f"audio_path: {self.audio_path}, "
            f"local_storage_base_dir: {self.local_storage_base_dir}, "
            f"local_results_base_dir: {self.local_results_base_dir} "
            f")"
        )


_CYBERPUNK_CONFIG: Optional[CyberpunkConfig] = None


def configure_config(provided_config: Optional[CyberpunkConfig] = None):
    global _CYBERPUNK_CONFIG

    if provided_config is not None:
        _CYBERPUNK_CONFIG = provided_config
    else:
        _CYBERPUNK_CONFIG = CyberpunkConfig(
            audio_path=os.environ.get("AUDIO_PATH", "local"),
            local_storage_base_dir=os.environ.get(
                "LOCAL_STORAGE_BASE_DIR",
                "testdata/",
            ),
            local_results_base_dir=os.environ.get(
                "LOCAL_RESULTS_BASE_DIR",
                None,
            ),
            s3_loader_bucket=os.environ.get("S3_LOADER_BUCKET", None),
            s3_loader_base_dir=os.environ.get("S3_LOADER_BASE_DIR", None),
            s3_storage_bucket=os.environ.get("S3_STORAGE_BUCKET", None),
            s3_storage_base_dir=os.environ.get("S3_STORAGE_BASE_DIR", None),
            s3_results_bucket=os.environ.get("S3_RESULTS_BUCKET", None),
            s3_results_base_dir=os.environ.get("S3_RESULTS_BASE_DIR", None),
        )


def get_config() -> CyberpunkConfig:
    global _CYBERPUNK_CONFIG

    if _CYBERPUNK_CONFIG is None:
        configure_config()

    assert _CYBERPUNK_CONFIG is not None
    return _CYBERPUNK_CONFIG



import logging
import logging.config
from typing import Generator, Optional

from flask import (
    Flask,
    Response,
    jsonify,
    make_response,
    request,
    stream_with_context,
)

from cyberpunk.config import CyberpunkConfig, configure_config
from cyberpunk.logger_config import LoggerConfig
from cyberpunk.processing import parse_query, process_args
from cyberpunk.storage import configure_storage

# from musicnn.tagger import top_tags


def create_app(cyberpunk_config: Optional[CyberpunkConfig] = None):
    configure_config(provided_config=cyberpunk_config)
    configure_storage()

    app = Flask(__name__)

    # 'always' (default), 'never',  'production', 'debug'
    app.config["LOGGER_HANDLER_POLICY"] = "always"

    # define which logger to use for Flask
    app.config["LOGGER_NAME"] = "cyberpunk"

    #  initialise logger
    app.logger

    logging.config.dictConfig(LoggerConfig.dictConfig)

    def stream_audio_file(filename: str, chunk_size: int = 2048) -> Generator:
        with open(f"/tmp/{filename}", "rb") as faudio:
            data = faudio.read(chunk_size)
            while data:
                yield data
                data = faudio.read(chunk_size)

    @app.route("/")
    def hello():
        return "Hello World"

    @app.route("/health")
    def healthcheck():
        return 200

    @app.route("/params/<path:filename>", methods=["GET"])
    def params_route(filename: str):
        """
        Route to format URL parameters as
        json to validate them before sending
        them to the `unsafe_processing` route
        """
        return jsonify(parse_query(filename, request.args))

    @app.route("/unsafe/<filename>", methods=["GET"])
    def unsafe_processing(filename: str):
        """
        Route to run processing pipeline on an audio file

        It's considered unsafe because there's currently no authentication or validation
        """
        args = request.args
        logging.critical(f"file path: {filename}, args: {args}")
        processed_file, file_type = process_args(filename, args)

        return Response(
            stream_with_context(stream_audio_file(processed_file)),
            mimetype=file_type,
        )

    @app.route("/unsafe/https://<path:url>", methods=["GET"])
    def unsafe_https_processing(url: str):
        """
        Route to run processing pipeline on an audio file

        It's considered unsafe because there's currently no authentication or validation
        """
        args = request.args
        logging.critical(f"file path: {url}, args: {args}")
        processed_file, file_type = process_args(f"https://{url}", args)

        return Response(
            stream_with_context(stream_audio_file(processed_file)),
            mimetype=file_type,
        )

    @app.route("/unsafe/http://<path:url>", methods=["GET"])
    def unsafe_http_processing(url: str):
        """
        Route to run processing pipeline on an audio file

        It's considered unsafe because there's currently no authentication or validation
        """
        args = request.args
        logging.critical(f"file path: {url}, args: {args}")
        processed_file, file_type = process_args(f"http://{url}", args)

        return Response(
            stream_with_context(stream_audio_file(processed_file)),
            mimetype=file_type,
        )

    @app.route("/tag/<filename>", methods=["GET"])
    def tag_audio_route(filename: str):
        """
        Route to get tags from audio files
        """

        # features = top_tags(f'./testdata/{filename}.mp3', model='MTT_musicnn', topN=10)
        features = [
            "hiphop",
            "drums",
            "fast",
        ]

        return {
            "file_key": filename,
            "tags": features,
        }

    @app.route("/stats", methods=["GET"])
    def storage_stats_route():
        """
        Route to get state on the audio store backend

        What's returned will depend on the audio store configured (local, s3, audius)
        """
        return {
            "tracks": 13019,
            "total time": "4.9 weeks",
            "total size": "71.1 GB",
            "artists": 548,
            "albums": 1094,
        }

    @app.errorhandler(404)
    def not_found(error):
        """Page not found."""
        print(error)
        print(request.url)
        print(request.args)
        return make_response("Route not found", 404)

    return app



from typing import Any, Dict, Optional

from pydub import AudioSegment

from cyberpunk.exceptions import (
    TransformationInputParseException,
    TransformationProcessException,
)


class Slice:
    def __call__(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        return self.process(segment, inputs)

    def parse_input_from_str(self, arg: str) -> Dict[str, Optional[int]]:

        try:
            start_str, end_str = tuple(arg.split(":"))

            start = int(start_str) if start_str != "" else None
            end = int(end_str) if end_str != "" else None

        except Exception as e:
            raise TransformationInputParseException()

        else:
            return {
                "start": start,
                "end": end,
            }

    def process(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:

        try:
            start = inputs["start"]
            end = inputs["end"]

            if start is None and end is None:
                raise TransformationProcessException(
                    "the start and end of a slice can't both be None",
                )

            if start is None:
                sliced_segment = segment[:end]
            elif end is None:
                sliced_segment = segment[start:]
            else:
                sliced_segment = segment[start:end]
        except Exception as e:
            raise TransformationProcessException(e)

        else:
            return sliced_segment



from typing import Any, Dict

from pydub import AudioSegment

from cyberpunk.exceptions import (
    TransformationInputParseException,
    TransformationProcessException,
)


class Reverse:
    def __call__(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        return self.process(segment, inputs)

    def parse_input_from_str(self, arg: str) -> Dict:

        try:
            if arg not in ["True", "true", "1", "yes", "Y", "y"]:
                return {"reverse": False}
        except Exception as e:
            raise TransformationInputParseException(e)
        else:
            return {"reverse": True}

    def process(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        try:
            if inputs["reverse"]:
                reversed_segment = segment.reverse()
            else:
                reversed_segment = segment
        except Exception as e:
            raise TransformationProcessException(e)
        else:
            return reversed_segment



from typing import Any, Dict

from pydub import AudioSegment

from cyberpunk.exceptions import (
    TransformationInputParseException,
    TransformationProcessException,
)
from cyberpunk.storage import get_storage


class Concat:
    def __call__(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        return self.process(segment, inputs)

    def parse_input_from_str(self, arg: str) -> Dict:
        other_filename = arg

        try:
            other_segment = get_storage().get_segment(other_filename)

        except Exception as e:
            raise TransformationInputParseException(e)
        else:
            return {
                "other": other_segment,
            }

    def process(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        try:
            other = inputs["other"]
            concated_segment = segment + other
        except Exception as e:
            raise TransformationProcessException(e)
        else:
            return concated_segment



"""Module containing various supported audio stores"""

import logging
from typing import Dict, Optional, Protocol, Tuple, Type

from pydub import AudioSegment

from cyberpunk.config import get_config
from cyberpunk.storage.audius import AudiusStorage
from cyberpunk.storage.http import HttpLoader
from cyberpunk.storage.local import LocalStorage
from cyberpunk.storage.s3 import S3Storage


class AudioStorageProtocol(Protocol):
    def __init__(self):
        """Declare variables like base dir"""

    def __contains__(self, element):
        """same as contains"""

    def contains(self, key: str) -> bool:
        """checks if a given key is in the audio store"""

    def get_segment(self, key: str) -> Tuple[AudioSegment, str]:
        """get an audio segment from storage"""


class AudioStorage:
    def __init__(
        self,
        audio_path: str,
        local_storage_base_dir: Optional[str] = None,
        local_results_base_dir: Optional[str] = None,
        s3_loader_bucket: Optional[str] = None,
        s3_loader_base_dir: Optional[str] = None,
        s3_storage_bucket: Optional[str] = None,
        s3_storage_base_dir: Optional[str] = None,
        s3_results_bucket: Optional[str] = None,
        s3_results_base_dir: Optional[str] = None,
    ):
        """Declare variables like base dir"""

        self.http_loader: HttpLoader = HttpLoader()
        self.storage_table: Dict[str, Type[AudioStorageProtocol]] = {
            "local": LocalStorage,
            "s3": S3Storage,
            "audius": AudiusStorage,
        }

        # local:s3:audius => [LocalStorage(), S3Storage(), AudiusStorage()]
        self.audio_path = list(
            map(
                lambda x: self.storage_table[x](),
                audio_path.split(":"),
            ),
        )

    def __str__(self):
        return ""

    def __repr__(self):
        return ""

    def __contains__(self, element):
        return self.contains(element)

    def contains(self, key: str) -> bool:
        """checks if a given key is in the audio store"""

        for storage in self.audio_path:
            if key in storage:
                return True

        return False

    def get_segment(self, key: str) -> Tuple[AudioSegment, str]:
        """get an audio segment from storage"""

        if key.startswith("https://") or key.startswith("http://"):
            return self.http_loader.get_segment(key)

        for storage in self.audio_path:
            if storage.contains(key):
                return storage.get_segment(key)

        raise KeyError(
            f"key `{key}` not found in any configured audio store ({self.audio_path})",
        )

    def save_segment(
        self,
        base_filename: str,
        segment: AudioSegment,
        file_format: str,
    ) -> str:
        """save an audio segment to storage and return the link to it"""

        # TODO: export with Filename unique to the stages run (for caching)
        processed_filename = f"processed_{base_filename}.{file_format}"
        segment.export(
            f"/tmp/{processed_filename}",
            format=file_format,
        )

        return processed_filename


# Audio Storage Singleton
_AUDIO_STORAGE: Optional[AudioStorage] = None


def configure_storage():
    global _AUDIO_STORAGE

    config = get_config()
    assert config is not None

    logging.info(f"configuring audio store")

    _AUDIO_STORAGE = AudioStorage(
        audio_path=config.audio_path,
        local_storage_base_dir=config.local_storage_base_dir,
        local_results_base_dir=config.local_results_base_dir,
        s3_loader_bucket=config.s3_loader_bucket,
        s3_loader_base_dir=config.s3_loader_base_dir,
        s3_storage_bucket=config.s3_storage_bucket,
        s3_storage_base_dir=config.s3_storage_base_dir,
        s3_results_bucket=config.s3_results_bucket,
        s3_results_base_dir=config.s3_results_base_dir,
    )


def get_storage() -> AudioStorage:
    global _AUDIO_STORAGE

    if _AUDIO_STORAGE is None:
        configure_storage()

    assert _AUDIO_STORAGE is not None
    return _AUDIO_STORAGE



from typing import Any, Dict

from pydub import AudioSegment

from cyberpunk.exceptions import (
    TransformationInputParseException,
    TransformationProcessException,
)


class Repeat:
    def __call__(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        return self.process(segment, inputs)

    def parse_input_from_str(self, arg: str) -> Dict[str, Any]:
        try:
            multiplier = int(arg)
        except Exception as e:
            raise TransformationInputParseException(e)
        else:
            return {
                "multiplier": multiplier,
            }

    def process(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:

        try:
            multiplier = inputs["multiplier"]
            repeated_segment = segment * multiplier
        except Exception as e:
            raise TransformationProcessException(e)
        else:
            return repeated_segment



import logging
from typing import Tuple

import boto3
from pydub import AudioSegment

from cyberpunk.config import get_config


class S3Storage:
    def __init__(self):
        config = get_config()

        self.s3 = boto3.client("s3")

        self.s3_loader_bucket = config.s3_loader_bucket
        self.s3_loader_base_dir = config.s3_loader_base_dir

        self.s3_storage_bucket = config.s3_storage_bucket
        self.s3_storage_base_dir = config.s3_storage_base_dir

        self.s3_results_bucket = config.s3_results_bucket
        self.s3_results_base_dir = config.s3_results_base_dir

    def __contains__(self, element):
        return self.contains(element)

    def contains(self, key: str) -> bool:
        return False

    def get_segment(self, key: str) -> Tuple[AudioSegment, str]:
        logging.info(f"pulling key from aws s3: {key}")

        self.s3.download_file(
            self.s3_storage_bucket,
            f"{self.s3_storage_base_dir}{key}",
            f"testdata/{key}",
        )

        segment = AudioSegment.from_file(f"testdata/{key}")

        return segment, f"testdata/{key}"



"""module for audio transformations"""

# nopycln: file

from typing import Any, Dict, Protocol

from pydub import AudioSegment

from cyberpunk.transformations.concat import Concat
from cyberpunk.transformations.fade_in import FadeIn
from cyberpunk.transformations.fade_out import FadeOut
from cyberpunk.transformations.repeat import Repeat
from cyberpunk.transformations.reverse import Reverse
from cyberpunk.transformations.slice import Slice


class Transformation(Protocol):
    def __call__(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        """runs the tranformation on the given AudioSegment and returns the output"""

    def parse_input_from_str(self, arg: str) -> Dict[str, Any]:
        """parses a string input proper inputs for that transformation"""

    def process(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        """runs the transformation on the given AudioSegment and returns the output"""



class LoggerConfig:
    dictConfig = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "standard": {
                "format": "%(asctime)s - %(name)s - %(levelname)s - "
                "%(message)s - [in %(pathname)s:%(lineno)d]",
            },
            "short": {"format": "%(message)s"},
        },
        "handlers": {
            "default": {
                "level": "INFO",
                "formatter": "standard",
                "class": "logging.StreamHandler",
            },
            "debug": {
                "level": "DEBUG",
                "formatter": "standard",
                "class": "logging.StreamHandler",
            },
        },
        "loggers": {
            "cyberpunk": {
                "handlers": ["default"],
                "level": "DEBUG",
                "propagate": True,
            },
            "werkzeug": {"propagate": True},
        },
        # 'root': { 'level': 'DEBUG', 'handlers': ['console'] }
    }



import logging
from typing import Tuple

import requests
from pydub import AudioSegment


class HttpLoader:
    def __init__(self):
        """Declare variables like base dir"""

    def get_segment(self, key: str) -> Tuple[AudioSegment, str]:
        """get an audio segment from storage"""
        logging.info(f"downloading audio file: {key}")
        req = requests.get(key)

        ext = key.split(".")[-1]

        # TODO: there must be a more robust way of saving temperary files like this
        location: str = f"remote-audio.{ext}"
        with open(f"testdata/{location}", "wb") as tmp_file:
            tmp_file.write(req.content)

        audio_segment = AudioSegment.from_file(
            f"testdata/{location}",
        )

        return audio_segment, location



import logging
import random
from typing import Tuple

import requests
from pydub import AudioSegment


class AudiusStorage:
    def __init__(self):
        self.host = random.choice(
            (requests.get("https://api.audius.co")).json()["data"],
        )

    def __contains__(self, element):
        return self.contains(element)

    def contains(self, key: str) -> bool:
        return False

    def get_segment(self, key: str) -> Tuple[AudioSegment, str]:
        logging.info(f"pulling key from audius: {key}")
        req = requests.get(
            f"{self.host}/v1/tracks/{key}/stream",
            params={"app_name": "cyberpunk"},
        )

        # TODO: there must be a more robust way of saving temperary files like this
        with open(f"testdata/{key}.mp3", "wb") as tmp_file:
            tmp_file.write(req.content)

        segment = AudioSegment.from_file(f"testdata/{key}.mp3")

        return segment, f"{key}.mp3"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cyberpunk', 'cyberpunk.storage', 'cyberpunk.transformations']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.0.2,<3.0.0',
 'PyYAML>=6.0,<7.0',
 'boto3>=1.20.54,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'gunicorn>=20.1.0,<21.0.0',
 'pydub>=0.25.1,<0.26.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['cyberpunk = main:main']}

setup_kwargs = {
    'name': 'cyberpunk',
    'version': '0.2.3',
    'description': 'Audio Processing Server',
    'long_description': '\n# Cyberpunk\n\nAudio Processing Server\n\n![GitHub](https://img.shields.io/github/license/jonaylor89/cyberpunk?logo=MIT) ![GitHub Workflow Status](https://img.shields.io/github/workflow/status/jonaylor89/cyberpunk/Docker)\n\n[![Run on Google Cloud](https://deploy.cloud.run/button.svg)](https://deploy.cloud.run?git_repo=https://github.com/jonaylor89/cyberpunk)\n\n\n### Quick Start\n\n```sh\ndocker run -p 8080:8080 -e PORT=8080 ghcr.io/jonaylor89/cyberpunk:main\n```\n\nOriginal audio:\n```sh\nhttps://raw.githubusercontent.com/jonaylor89/cyberpunk/main/testdata/celtic_pt2.mp3\n```\n\n![](testdata/celtic_pt2.mp3)\n\n\nTry out the following audio URLs:\n```\nhttp://localhost:8000/unsafe/https://raw.githubusercontent.com/jonaylor89/cyberpunk/main/testdata/celtic_pt2.mp3\nhttp://localhost:8000/unsafe/https://raw.githubusercontent.com/jonaylor89/cyberpunk/main/testdata/celtic_pt2.mp3?reverse=true\nhttp://localhost:8000/unsafe/https://raw.githubusercontent.com/jonaylor89/cyberpunk/main/testdata/celtic_pt2.mp3?slice=0:10000\nhttp://localhost:8000/unsafe/https://raw.githubusercontent.com/jonaylor89/cyberpunk/main/testdata/celtic_pt2.mp3?reverse=true&repeat=1&slice=1000:5000\n\n```\n\n### Cyberpunk Endpoint\n\nCyberpunk endpoint is a series of URL parts which defines the image operations, followed by the image URI:\n\n```\n/HASH|unsafe/AUDIO?slice&concat&fade_in&fade_out&repeat&reverse&filters=NAME(ARGS)\n```\n\n- `HASH` is the URL Signature hash, or `unsafe` if unsafe mode is used\n- `slice`\n- `concat`\n- `fade_in`\n- `fade_out`\n- `repeat`\n- `reverse`\n- `AUDIO` is the audio URI\n\n\nCyberpunk provides utilities for previewing and generating Cyberpunk endpoint URI, including the [cyberpunk_path](https://github.com/jonaylor89/cyberpunk/tree/main/cyberpunk/processing.py) function and the `/params` endpoint:\n\n#### `GET /params`\n\nPrepending `/params` to the existing endpoint returns the endpoint attributes in JSON form, useful for preview:\n\n```sh\ncurl "http://localhost:8000/unsafe/celtic_p2.mp3?reverse=true&repeat=1&slice=1000:5000"\n\n{\n  "path": "/celtic_p2.mp3?reverse=true&repeat=1&slice=1000:5000"\n  "audio": "celtic_pt2.mp3",\n  "hash": "=",\n  "reverse": true,\n  "repeat": 1,\n  "slice": {\n      "start": 1000,\n      "end": 5000,\n  }\n}\n```\n\n# Features\n\n- [x] Audio Streaming\n\n- [x] Change encodings (e.g. mp3 -> wav)\n- [x] Audio slicing\n- [ ] Change Volume\n- [x] Concat Audio\n- [x] Repeat Audio\n- [x] Reverse Audio\n- [ ] Crossfade\n- [x] Fade in/out\n- [ ] Audio Quality\n- [ ] Audio Tagging\n- [ ] Audio Thumbnails\n- [ ] Mastering Music\n\n- [ ] Sound/Vocal Isolation\n\n- [ ] [Cool ML Stuff](https://github.com/spotify/pedalboard)\n\n- [ ] [File Caching](https://gist.github.com/ruanbekker/75d98a0d5cab5d6a562c70b4be5ba86d)\n\n# Storage Options\n\n- [x] Local\n- [ ] Cloud (e.g. S3)\n- [x] Blockchain (Audius)\n\n\n# Environment\n\nPORT: 8080 # server port number\n\nCYBERPUNK_SECRET: mysecret # secret key for URL signature\n\nAWS_ACCESS_KEY_ID: ...\n\nAWS_SECRET_ACCESS_KEY: ...\n\nAWS_REGION: us-east-1\n\nAUDIO_PATH=local:s3:audius:http\n\nSTORAGE_BASE_DIR=testdata/\n\nRESULTS_STORAGE_BASE_DIR=processed/\n\nS3_LOADER_BUCKET=mybucket\n\nS3_LOADER_BASE_DIR=audio/\n\nS3_STORAGE_BUCKET=mybucket\n\nS3_STORAGE_BASE_DIR=audio/\n\nS3_RESULTS_STORAGE_BUCKET=mybucket\n\nS3_RESULTS_STORAGE_BASE_DIR=audio/results\n\n\n# Docker Compose Example\n\nCyberpunk with file system, using mounted volumn:\n\n```yaml\nversion: "3"\nservices:\n  imagor:\n    image: jonaylor/cyberpunk:main\n    volumes:\n      - ./:/mnt/data\n    environment:\n      PORT: 8080\n      AUDIO_PATH: "local"\n      FILE_STORAGE_BASE_DIR: /mnt/data/testdata/ # enable file storage by specifying base dir\n    ports:\n      - "8080:8080"\n```\n\nCyberpunk with AWS S3:\n\n```yaml\nversion: "3"\nservices:\n  imagor:\n    image: jonaylor/cyberpunk:main\n    environment:\n      PORT: 8080\n      CYBERPUNK_SECRET: mysecret # secret key for URL signature\n      AWS_ACCESS_KEY_ID: ...\n      AWS_SECRET_ACCESS_KEY: ...\n      AWS_REGION: ...\n\n      AUDIO_PATH: "s3"\n\n      S3_LOADER_BUCKET: mybucket # enable S3 loader by specifying bucket\n      S3_LOADER_BASE_DIR: audio # optional\n\n      S3_STORAGE_BUCKET: mybucket # enable S3 storage by specifying bucket\n      S3_STORAGE_BASE_DIR: audio # optional\n\n      S3_RESULT_STORAGE_BUCKET: mybucket # enable S3 result storage by specifying bucket\n      S3_RESULT_STORAGE_BASE_DIR: audio/result # optional\n    ports:\n      - "8080:8080"\n```',
    'author': 'Johannes Naylor',
    'author_email': 'jonaylor89@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jonaylor89/cyberpunk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict

from pydub import AudioSegment

from cyberpunk.exceptions import (
    TransformationInputParseException,
    TransformationProcessException,
)


class FadeIn:
    def __call__(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        return self.process(segment, inputs)

    def parse_input_from_str(self, arg: str) -> Dict:
        try:
            duration = int(arg)
        except Exception as e:
            raise TransformationInputParseException(e)
        else:
            return {
                "duration": duration,
            }

    def process(
        self,
        segment: AudioSegment,
        inputs: Dict[str, Any],
    ) -> AudioSegment:
        try:
            duration = inputs["duration"]
            faded_segment = segment.fade_in(duration)

        except Exception as e:
            raise TransformationProcessException(e)
        else:
            return faded_segment



import logging
from typing import Any, Dict, List, Tuple

from cyberpunk.storage import get_storage
from cyberpunk.transformations import (
    Concat,
    FadeIn,
    FadeOut,
    Repeat,
    Reverse,
    Slice,
    Transformation,
)


def process_args(base_filename: str, args: Dict) -> Tuple[str, str]:

    lookup_table: Dict[str, Transformation] = {
        "reverse": Reverse(),
        "repeat": Repeat(),
        "slice": Slice(),
        "concat": Concat(),
        "fade_in": FadeIn(),
        "fade_out": FadeOut(),
    }

    supported_formats: List[str] = [
        "mp3",
        "wav",
        "flac",
    ]

    # Create Audio Segment
    audio_segment, tmp_location = get_storage().get_segment(base_filename)

    # Pass Audio Segment through Each Stage
    for (k, v) in args.items():
        if k in lookup_table.keys():

            logging.info(f"running transformation: {k}")

            transformation: Transformation = lookup_table[k]
            assert transformation is not None

            try:
                inputs: Dict[str, Any] = transformation.parse_input_from_str(v)
            except Exception as e:
                logging.error(f"failure to parse input `{v}` for `{k}` : {e}")
                continue

            try:
                audio_segment = transformation.process(audio_segment, inputs)
            except Exception as e:
                logging.error(
                    f"failure to process input `{v}` for `{k}` : {e}",
                )
                continue

    # TODO this can be significantly better
    file_format = "mp3"
    if "format" in args.keys():
        file_format = (
            args["format"] if args["format"] in supported_formats else "mp3"
        )

    processed_filename = get_storage().save_segment(
        tmp_location,
        audio_segment,
        file_format,
    )

    # Return Filename and Audio Type
    return processed_filename, f"audio/{file_format}"


def parse_query(filename: str, args: Dict) -> Dict:
    """
    Parse and generate a Python object based on a cyberpunk endpoint
    """
    return {"audio": filename, **args}


def cyberpunk_path():
    """
    Parse and generate a cyberpunk endpoint based on a Python object
    """
    return
